@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 from igo.dictionary import Matrix, WordDic, Unknown, ViterbiNode
-from igo.util import UTF16Codec
+from igo.dictreader import UTF16Codec
 import os.path
 from os.path import dirname, abspath
 import array
@@ -19,6 +19,15 @@ class Morpheme:
         self.start = start
         """ テキスト内での形態素の出現開始位置 """
 
+    def __str__(self):
+        return self.fmt()
+
+    def fmt(self,
+            fmt="surface: {surface}, feature: {feature}, start={start}"):
+        return fmt.format(surface=self.surface,
+                          feature=self.feature,
+                          start=self.start)
+
 
 class Tagger:
     """
@@ -37,19 +46,17 @@ class Tagger:
             return path
         return None
 
-    def __init__(self, dataDir=None, gae=False):
+    def __init__(self, path=None, gae=False, use_mmap=None):
         """
         バイナリ辞書を読み込んで、形態素解析器のインスタンスを作成する
 
-        @param dataDir バイナリ辞書があるディレクトリ
-        @throws FileNotFoundException 間f違ったディレクトリが指定された場合に送出される
-        @throws IOException その他の入出力エラーが発生した場合に送出される
+        @param path directory of a binary dictionary
         """
-        if not dataDir:
-            dataDir = Tagger.lookup()
-        self.wdc = WordDic(dataDir, gae, gae)
-        self.unk = Unknown(dataDir, gae)
-        self.mtx = Matrix(dataDir, gae)
+        if not path:
+            path = Tagger.lookup()
+        self.wdc = WordDic(path, gae, gae, use_mmap)
+        self.unk = Unknown(path, gae, use_mmap)
+        self.mtx = Matrix(path, gae, use_mmap)
 
     def parse(self, text, result=None):
         """
@@ -65,7 +72,7 @@ class Tagger:
         wordData = self.wdc.wordData
         while vn:
             surface = text[vn.start:vn.start + vn.length]
-            feature = UTF16Codec.decode(wordData(vn.wordId).tostring())[0]
+            feature = UTF16Codec.decode(wordData(vn.wordId))[0]
             result.append(Morpheme(surface, feature, vn.start))
             vn = vn.prev
         return result
@@ -129,6 +136,17 @@ class Tagger:
         vn.cost += minCost
         return vn
 
+    def release(self):
+        self.wdc.release()
+        self.unk.release()
+        self.mtx.release()
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, et, ev, tb):
+        self.release()
+
 
 class MakeLattice:
     def __init__(self, nodesAry, setMincostNode):
